@import (less) "../node_modules/bootstrap-less/bootstrap/variables.less";
@import (less) "../node_modules/patternfly/dist/less/variables.less";

@metadata-color: #888;

@listing-ct-hover: @list-group-hover-bg;
@listing-ct-active: #65bedf;
@listing-ct-padding: 10px;
@listing-ct-spacing: 15px;
@listing-ct-open: #f5f5f5;
@listing-ct-open-width: 3px;
@listing-ct-metadata: @metadata-color;
@listing-ct-warning-color: #fbc7c7;
@listing-ct-border: #ccc;
@listing-ct-border-light: #eee;
@listing-ct-border-maybe: #ddd;

@screen-lg-max: (@screen-xlg-min - 1);
@screen-xlg-min: 1600px;
@screen-xs: 480px;
@screen-xs-min: @screen-xs;
@screen-xxs-max: (@screen-xs-min - 1);
