#cockpit_modal_dialog .modal-body.scroll {
    max-height: calc(75vh - 10rem);
    overflow: auto;
}

.dialog-wait-ct {
    margin-top: 3px;
}

.dialog-wait-ct .spinner {
    display: inline-block;
}

.dialog-wait-ct span {
    vertical-align: 4px;
    padding-left: 10px;
}

.modal-content .alert {
    text-align: left;
    padding-top: 5px;
    padding-bottom: 5px;
}

.modal-content .alert .fa {
    position: absolute;
    left: 10px;
    top: 6px;
    font-size: 20px;
}

.modal-content .alert .pficon {
    top: 5px;
}

.alert.alert-danger .fa {
    color: #af151a;
}
